/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*****************************************************************

    c   o   n   t   e   x   t   .   c   p   p

*****************************************************************/

#include "context.h"
#include "tree.h"

/*****************************************************************
    C L i s t   methods
*****************************************************************/

CList::CList()
{
    refCount = 1;
}

void CList::incRefCount()
{
    refCount++;
}

int CList::decRefCount()
{
    return --refCount;
}

/*****************************************************************

    C o n t e x t    methods

******************************************************************/

Context::Context()
{
    array = new CList;
    position = -1;
}

Context::~Context()
{
    assert(array);
    if (!array -> decRefCount())
        delete array;
}

Vertex *Context::current() const
{
    if (!isFinished())
        return (*array)[position];
    else
        return NULL;
};

void Context::reset()
{
    if (!array -> number())
        position = -1;
    else
        position = 0;
}

Bool Context::isFinished() const
{
    return (Bool) !((position >= 0) && (position < array -> number()));
}

Bool Context::isVoid() const
{
    return (Bool) !(array -> number());
}

Vertex *Context::shift()
{
    if ((position >= 0) && (position < number() - 1))
        return (*this)[++position];
    position = -1;
    return NULL;
}

void Context::set(Vertex *v)
{
    array -> append(v);
    reset();
}

Vertex *Context::operator[] (int n) const
{
    return (*array)[n];
}

void Context::deppendall()
{
    if (!array -> decRefCount())
        delete array;
    array = new CList;
    position = -1;
}

void Context::append(Vertex *v)
{
    array -> append(v);
    reset();
}

void Context::deppend()
{
    array -> deppend();
    if (position >= array -> number())
        position = array -> number() - 1;
}

int Context::number() const
{
    return array -> number();
}

Context* Context::copy()
{
    Context *newc = new Context;
    delete NZ(newc -> array);
    newc -> array = array;
    array -> incRefCount();
    newc -> reset();
    return newc;
}

void Context::swap(int i, int j)
{
    array -> swap(i, j);
}

Context *Context::swallow(Context *other)
{
    Context *result = new Context;
    int i = 0, j = 0;
    long vstamp, wstamp;
    Vertex *v, *w;
    while ((i < array -> number()) && (j < other -> array -> number()))
    {
        vstamp = (v = (*array)[i]) -> stamp;
        wstamp = (w = (*(other -> array))[j]) -> stamp;
        if (vstamp == wstamp)
        {
            j++;
            continue;
        }
        if (vstamp < wstamp)
        {
            result -> append(v);
            i++;
        }
        else
        {
            result -> append(w);
            j++;
        }
    }
    while (i < array -> number())
        result -> append((*array)[i++]);
    while (j < other -> array -> number())
        result -> append((*(other -> array))[j++]);
    deppendall();
    other -> deppendall();
    return result;
}
