/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#include "datastr.h"
#include "verts.h"
#include <ctype.h>
#include <malloc.h>

Str theEmptyString;

//definitions for Stack and List are in datastr.h
//as they are templates

/****************************************
S t r
****************************************/

Str::Str()
:
/* _string(NULL), */_parts(1)
{
    *this = "";
}

Str::Str(char c)
:
/*_string(NULL), */_parts(1)
{
    *this = c;
}

Str::Str(char* s)
:
/*_string(NULL), */_parts(1)
{
    *this = s;
}

Str::Str(const Str& other)
{
    operator=((char*)(Str&)other);
    //*this = other;
};

Str::~Str()
{
    remove();
}

BOOL Str::isEmpty() const
{
    return _length ? FALSE : TRUE;
}

Bool Str::toDouble(double &d)
{
    char *stopper;
    compact();
    d = strtod(operator char*(),&stopper);
    return (Bool) (*stopper != 0);
}

void Str::empty()
{
    (*this) = "";
}

Str::operator char* ()
{
    compact();
    return _parts[0];
}

Str& Str::operator= (char* text)
{
    if (!text)
        empty();
    else 
    {
        char *p;
        remove();
        _parts.append( p = new char[(_length = strlen(text)) + 1] );
        strcpy(p,text);
    };
    return *this;
}

Str& Str::operator= (int num)
{
    char buf[20];
    sprintf(buf,"%d",num);
    (*this) = buf;
    return *this;
}

Str& Str::operator= (double num)
{
    char buf[20];
    sprintf(buf,"%g",num);
    (*this) = buf;
    return *this;
}

Str& Str::operator= (const Str &other)
{
    operator=((char*)(Str&)other);
    return *this;
}

Str& Str::operator= (char c)
{
    char *p;
    remove();
    _length = 1;
    _parts.append( p = new char[2] );
    *p = c;
    p[1] = 0;
    return *this;
}

char Str::operator[] (int i)
{
    assert((i >= 0) && (i <= _length));
    compact();
    return _parts[0][i];
}

Bool Str::eqNoCase (const char* other)
{
    assert(_parts.number() > 0);
    if (_parts.number() > 1) 
        compact();
    return (strEqNoCase(_parts[0], other) ? TRUE : FALSE);
}

BOOL Str::operator== (const char* other)
{
    assert(_parts.number() > 0);
    if (_parts.number() > 1) 
        compact();
    return (strcmp(_parts[0], other) ? FALSE : TRUE);
}

Bool Str::operator== (char* other)
{
  return operator==((const char*) other);
}

Bool Str::operator== (Str &other)
{
  return operator==((const char*) other);
}

int Str::compare(char* other)
{
    assert(_parts.number() > 0);
    if (_parts.number() > 1) 
        compact();
    return (strcmp(_parts[0], other) < 0);
}

Bool Str::operator< (char* other)
{
    return (Bool) (compare(other) < 0);
}

Bool Str::operator< (Str &other)
{
  return operator<((char*) other);
}



Str& Str::operator+= (const char *adding)
{
    if (!*adding)
        return *this;
    int len = strlen(adding);
#ifdef WIN32
    char *lastone = _parts.last();
    long lastlen = strlen(lastone);
    if (_expand(lastone, lastlen + len + 1))
        strcpy(lastone + lastlen, adding);
    else
    {
#endif
        char *newpart;
        _parts.append(newpart = new char[len+1]);
        strcpy(newpart,adding);
#ifdef WIN32
    };
#endif
    _length += len;
    return *this;
}

Str& Str::operator+= (int addingnum)
{
    char buf[20];
/*
    int len = 
*/
    sprintf(buf,"%d",addingnum);
/*
    char* newpart = new char[len + 1];
    strcpy(newpart,buf);
    (*this) += newpart;
*/
    return (*this += buf);
}

Str& Str::operator+= (char addingc)
{
    char *newpart = new char[2];
    *newpart = addingc; 
    newpart[1] = 0;
    (*this) += newpart;
    delete newpart;
    return *this;
}

Str Str::operator+ (const char *adding)
{
    Str temp = *this;
    temp += adding;
    return temp;
}

Str Str::operator+ (Str &adding)
{
    return operator+((char*) adding);
}

Str Str::operator+ (int addingnum)
{
    *this += addingnum;
    return *this; //temp;
}

int Str::length()
{
    assert(_length >= 0);
    return _length;
}

Str& Str::add(char *t)
{
    assert(t);
    operator += (t);
//    compact();
    return *this;
}

Str& Str::nadd(const char *t, int len)
{
    assert(t);
    if (!len) return *this;
    char *newS = new char[len+1];
    if (len) strncpy(newS,t,len);
    newS[len] = 0;
    (*this) += newS;
    delete newS;
    return *this;
}

Str& Str::nset(const char *t, int len)
{
    empty();
    nadd(t,len);
    return *this;
}

void Str::speakTerse(Str &ret)
{
    int i;
    char c;
    compact();
    for (i = 0; i < length(); i++)
    {
        switch(c = (*this)[i])
        {
        case '\n': 
            ret += "&#10;"; break;
        case '\t': 
            ret += "&#9;"; break;
        default:
            ret += c;
        };
    };
}

// static function performing the same as isalnum() (so far).
// should be extended to handle UTF-8 and XML's requirements
// on valid name characters

Bool Str::namechar(char c)
{
    return (Bool) isalnum(c);
}

/*================================================================
isdigit_
does the same as ordinary isdigit so far.
================================================================*/

Bool Str::isdigit_(char c)
{
    return (Bool) isdigit(c);
}


// p r i v a t e

void Str::compact()
{
    int num = _parts.number();
    assert(num > 0);
    if (num == 1) return;

    char *p = new char[_length + 1];
    int i;
    *p = 0;
    for (i = 0; i < num; i++)
        strcat(p,_parts[i]);
    int waslength = _length;
    remove();
    _length = waslength;
    _parts.append(p);
}

void Str::remove()
{
    if (_parts.number())
        _parts.freeall(TRUE);
    _length = -1;
};

void Str::forget()
{
    _parts.deppendall();
    _length = -1;
}

/****************************************
Q N a m e
****************************************/

void QName::speak(Str &s, SpeakMode mode) const
{
    if (!prefix.isEmpty())
    {
        s += (char*)(Str&)prefix;
        s += ":";
    };
    s += (char*)(Str&)local;
}

Bool QName::prefixIsNS(Str &uri2)
{
    return (Bool) (uri.operator ==(uri2));
} 

eFlag QName::setFromExpat(const Str &s)
{
    char *p = (char *)(Str &) s,
        *q = strchr(p,NS_SEP);
    if (q)
    {
        *q = 0;
        uri = p;
        *q = NS_SEP;
        local = (q+1);
        if (strchr(q+1,':'))
	  {
		Str msg = "{";
		msg += uri + "}:";
            msg += local;
		Err1(E_EXTRA_COLON, (char *)msg);
        }
    }
    else
    {
	  uri.empty();
        local = p;
        char *isColon;
        if (!!(isColon = strchr(p,':')))
        {
            *isColon = 0;
            Err1(ET_BAD_PREFIX,p);
        }
        prefix.empty();
    };
    return OK;
}

eFlag QName::setLogical(const Str &s, NSList *resolver, Bool defaultToo)
{
    char *p = (char *)(Str &) s,
        *q = strchr(p,':'),
        *localPart;
    if (q)
    {
        *q = 0;
        prefix = p;
        *q = ':';
        localPart = (q+1);
    }
    else
    {
        prefix.empty();
        localPart = p;
    };
    NZ(resolver) -> resolve(uri = prefix, defaultToo);
    if (strchr(localPart,':'))
        Err1(E_EXTRA_COLON, (char *)(Str &)s)
    else
        local = localPart;
    return OK;
}

void QName::setPrefix(const Str& pref)
{
    prefix = pref;
}

void QName::findPrefix(const NSList& resolver)
{
    if (uri.isEmpty())
        prefix.empty();
    else
        resolver.unresolve(prefix = uri);
}


Bool QName::operator== (QName &other)
{
    return (Bool) ((local.operator==(other.local)) && 
		   (uri.operator==(other.uri)));
}

Str QName::getname()
{
    Str s;
    speak(s,SM_NAME);
    return s;
}

const Str& QName::getLocal() const
{
    return local;
}

const Str& QName::getUri() const
{
    return uri;
}


/*****************************************************************
    S t r S t r L i s t
*****************************************************************/

int StrStrList::findNum(const Str &_key)
{
    int i;  
    for (i = 0; (i < number()) && !(((Str&)_key).operator== ((*this)[i] -> key)); i++);
    return (i < number()) ? i : -1;
}

Str* StrStrList::find(Str &_key)
{
    int ndx = findNum(_key);
    return (ndx != -1) ? &((*this)[ndx] -> value) : NULL;
}

