/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <iostream.h>
#include <assert.h>
#include "error.h"
//#include "base.h"
//#include "proc.h"

SabMsg SablotMessages[] = 
{
    {E_OK, "OK"},
    {E_XML, "XML parser error %s: %s"}, 
    {E_MEMORY, "out of memory"},
    {E_FILE_OPEN, "cannot open file '%s'"}, 
    {E_BUFFER_SMALL, "output buffer too small"}, 
    {E_BOX_SET, "opening an open file/buffer"}, 
    {E_BOX_NOT_SET, "using a file/buffer that is not open"}, 
    {E_BOX_MODE, "file/buffer access mode violation"}, 
    {E_BUFFER_NOT_FREED, "allocated buffer has not been freed"}, 
    {E_BUFFER_TO_FREE, "buffer to be freed has not been allocated"}, 
    {E_IN_PROGRESS, "processing is in progress"}, 
    {ES_LEAF_ON_STACK, "leaf on stack"}, 
    {ES_LIST_INDEX, "list index out of bounds"}, 
    {ES_CAST, "dynamic cast failed"}, 
    {ES_GET_DELIM, "error in getDelim()"}, 
    {ES_VERTEX_EXECUTE, "Vertex::execute() called"}, 
    {ET_UNSUPP_XSL, "unsupported XSL instruction"}, 
    {ET_BAD_XSL, "non-XSL instruction"}, 
    {ET_REQ_ATTR, "missing required XSLT attribute"}, 
    {ET_BAD_ATTR, "unexpected attribute '%s'"},
    {ET_OPEN_QUOTES, "odd quote"}, 
    {ET_COLON_STARTS_LOC_STEP, "location step begins with :"}, 
    {ET_UNKNOWN_AXIS, "unknown axis"}, 
    {ET_EXPR_SYNTAX, "wrong expression syntax"}, 
    {ET_MISSING_LEFT, "missing opening delimiter in expression"}, 
    {ET_MISSING_RIGHT, "missing closing delimiter in expression"}, 
    {ET_BAD_NUMBER, "number in bad format"}, 
    {ET_LPAREN_EXP, "'(' expected"}, 
    {ET_RPAREN_EXP, "')' expected"}, 
    {ET_LPARCKET_EXP, "'(' or '[' expected"}, 
    {ET_RBRACKET_EXP, "']' expected"}, 
    {ET_UNKNOWN_NODETYPE, "call to unknown function"}, 
    {ET_FUNC_NOTALLOWED, "function not allowed here"}, 
    {ET_EMPTY_PATT, "pattern is empty"}, 
    {ET_BAD_TOKEN, "token '%s' not recognized"}, 
    {ET_LOCSTEP_END_EXP, "LocStep end expected"}, 
    {ET_BAD_AXIS_IN_PATTERN, "bad axis in pattern"}, 
    {ET_VARIABLE_NOT_FOUND, "variable not found"}, 
    {ET_XSL_SYNTAX, "XSLT syntax error"}, 
    {ET_BAD_PREFIX, "invalid namespace prefix '%s'"}, 
    {ET_DUPLICATE_RULE_NAME, "duplicate rule name '%s'"}, 
    {ET_RULE_NOT_FOUND, "called nonexistent rule"}, 
    {ET_CONTEXT_FOR_BAD_EXPR, "expression is not a node set"}, 
    {ET_FUNC_NOT_SUPPORTED, "function '%s' not supported"}, 
    {ET_MULT_ASSIGNMENT, "conflicting variable bindings"},
    {E_EXTRA_COLON, "name '%s' contains two colons"},
    {E_ELEM_MUST_EMPTY, "XSL element '%s' must be empty"},
    {E_ELEM_CONT_TEXT_OR_LRE, "XSL element '%s' can only contain XSL elements"},
    {E_ELEM_CONTAINS_ELEM, "XSL element '%s' cannot contain element '%s' at this point"},
    {E_BAD_ELEM_CONTENT, "bad element content"},
    {ET_BAD_ARGS_N, "wrong number of function arguments"},
    {ET_BAD_ARG_TYPE, "illegal argument type"},
    {E1_UNSUPPORTED_SCHEME, "unsupported URI scheme '%s'"},
    {E1_URI_OPEN, "could not open document '%s'"},
    {E1_URI_CLOSE, "could not close document '%s'"},
    {E1_ARG_NOT_FOUND,"'arg:%s' not found"},
    {E1_DUPLICATE_ARG, "duplicate argument '%s'"},
    {E1_URI_READ, "cannot read from URI '%s'"},
    {E1_URI_WRITE, "cannot write to URI '%s'"},
    {E1_CANNOT_WRITE, "cannot write to '%s'"},
    {E_ODD_PAIRS, "odd number of items in the argument name-value list"},

    {E_NONE, ""},
    {W_NO_CHERROR, "recompile with chError enabled -- using stderr"},
    {W_CHERROR_ONLY_ERR, "chError currently only supports errors -- using stderr"},
    {W_UNSUPP_LANG, "function lang() not supported, returning false"},
    {W_NO_STYLESHEET, "the included or imported stylesheet '%s' contains "\
        "no xsl:stylesheet or xsl:transform element"},
    {W_SH_REGISTERED, "conflicting registration of a scheme handler ignored"},
    {W_SH_NOT_REGISTERED, "cannot unregister scheme handler, none registered"},
    {W_NONE, ""},
    {L_START, "\n--------starting processing: %s"},
    {L_STOP, "\n--------stopping: %s"},
    {L_STARTING_PARSE, "parsing file '%s'"},
    {L_NONE, ""},
    {MSG_ERROR, "Error"},
    {MSG_WARNING, "Warning"},
    {MSG_LOG,"Log message"},
    {MSG_LAST, ""}
};


SabMsg* GetMessage(MsgCode e)
{
    SabMsg *p = SablotMessages;
    while ((p -> code != e) && (p -> code != MSG_LAST))
        p++;
    return p;
}
