/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef ErrorHIncl
#define ErrorHIncl

#define Err(code)   {situation.error(code, (char*) NULL, (char*) NULL); return NOT_OK;}
#define Err1(code, arg1) {situation.error(code, arg1, (char*) NULL); return NOT_OK;}
#define Err2(code, arg1, arg2) {situation.error(code, arg1, arg2); return NOT_OK;}
#define E(statement) {if (statement) return NOT_OK;}

#define Warn(code)  {situation.warning(code, (char*) NULL, (char*) NULL);}
#define Warn1(code, arg1) {situation.warning(code, arg1, (char*) NULL);}
#define Log(code)  {situation.logmsg(code, (char*) NULL, (char*) NULL);}
#define Log1(code, arg1) {situation.logmsg(code, arg1, (char*) NULL);}

enum MsgCode
{
    //no error:
    E_OK,
        //XML parser error:
        E_XML,
        //file problem:
        E_FILE_OPEN,
        //output buffer too small:
        E_BUFFER_SMALL,
        //not enough memory to open processor
        E_MEMORY,
        //box (file/buffer) already set (open)
        E_BOX_SET,
        //box (file/buffer) not set yet
        E_BOX_NOT_SET,
        //box mode violation
        E_BOX_MODE,
        E_BUFFER_NOT_FREED,
        E_BUFFER_TO_FREE,
        E_IN_PROGRESS,
        
        E_LOG_WRITE,
        E_LOG_OPEN,
        E_LOG_CLOSE,
        E_NEW_FAILED,
        E_DELETE_FAILED,

        E_BAD_QNAME,
        
        //ES = internal (Sablot) errors
        ES_ = 0x40,
        //a non-element found on the append stack:
        ES_LEAF_ON_STACK,
        //dynamic cast error:
        ES_CAST,
        //list index out of bounds:
        ES_LIST_INDEX,
        //error in getDelim()
        ES_GET_DELIM,
        //Vertex::execute() called
        ES_VERTEX_EXECUTE,
        
        //EI = input document errors
        EI_ = 0x80,
        
        //ET = template (stylesheet) errors
        ET_ = 0xc0,
        //unsupported XSL instruction:
        ET_UNSUPP_XSL,
        //nonexistent XSL instruction:
        ET_BAD_XSL,
        //missing required attribute
        ET_REQ_ATTR,
        ET_BAD_ATTR,
        ET_BAD_CHILDREN,
        //odd quote
        ET_OPEN_QUOTES,
        //location step starts with :
        ET_COLON_STARTS_LOC_STEP,
        //bad axis:
        ET_UNKNOWN_AXIS,
        //general expression syntax error:
        ET_EXPR_SYNTAX,
        //missing opening delimiter:
        ET_MISSING_LEFT,
        //missing closing delimiter:
        ET_MISSING_RIGHT,
        //number in bad format:
        ET_BAD_NUMBER,
        ET_BAD_VAR,
        ET_INFINITE_LITERAL,
        ET_OPERATOR_EXP,
        //'(' expected:
        ET_LPAREN_EXP,
        //')' expected:
        ET_RPAREN_EXP,
        //'(' or '[' expected:
        ET_LPARCKET_EXP,
        //']' expected:
        ET_RBRACKET_EXP,
        //invalid node type
        ET_UNKNOWN_NODETYPE,
        //node type not allowed
        ET_FUNC_NOTALLOWED,
        //pattern empty:
        ET_EMPTY_PATT,
        //bad token:
        ET_BAD_TOKEN,
        //LocStep end expected:
        ET_LOCSTEP_END_EXP,
        //patterns allow only several axes:
        ET_BAD_AXIS_IN_PATTERN,
        ET_BAD_PATTERN,
        //root not first in pattern:
        ET_PATT_ROOT_NOT_FIRST,
        ET_EXPR_NOT_NODESET,
        ET_CONTEXT_FOR_BAD_EXPR,
        ET_VARIABLE_NOT_FOUND,
        ET_BAD_ARGS_N,
        ET_BAD_ARG_TYPE,
        ET_FUNC_NOT_SUPPORTED,
        ET_XSL_SYNTAX,
        ET_BAD_PREFIX,
        ET_BAD_NS,
        ET_RULE_NOT_FOUND,
        ET_DUPLICATE_RULE_NAME,
        ET_MULT_ASSIGNMENT,
        E_EXTRA_COLON,
        E_ELEM_MUST_EMPTY,
        E_ELEM_CONT_TEXT_OR_LRE,
        E_ELEM_CONTAINS_ELEM,
        E_BAD_ELEM_CONTENT,
        E1_UNSUPPORTED_SCHEME,
        E1_URI_OPEN,
        E1_URI_CLOSE,
        E1_URI_READ,
        E1_URI_WRITE,
        E1_ARG_NOT_FOUND,
        E1_CANNOT_WRITE,
        E1_DUPLICATE_ARG,

        E_ODD_PAIRS,

        E_NONE,
        //mask to distinguish ES/EI/ET/ED
        E_MASK = 0x01c0,

        // warnings
        W_NO_CHERROR,
        W_CHERROR_ONLY_ERR,
        W_UNSUPP_LANG,
        W_NO_STYLESHEET,
        //  scheme handler warnings
        W_SH_REGISTERED,
        W_SH_NOT_REGISTERED,

        W_NONE,

        // log messages
        L_START,
        L_STOP,
        L_STARTING_PARSE,
        L_NONE,

        MSG_ERROR = 0x4000,
        MSG_WARNING,
        MSG_LOG,
        MSG_LAST
};

struct SabMsg
{
    MsgCode code;
    char *text;
};

extern SabMsg* GetMessage(MsgCode e);

#endif //ifndef ErrorHIncl


