/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef ProcHIncl
#define ProcHIncl

#include "base.h"
#include "datastr.h"
#include "uri.h"
// #include "vars.h"
//#include "expr.h"
//#include "context.h"

extern Str ns_XSL;

class Tree;
class LocStep;
class Vertex;
class XSLElement;
class Expression;
class Context;
typedef PList<Expression*> ExprList;

/*****************************************************************

    R u l e I t e m
    an item of RuleSList describing a rule

*****************************************************************/

class RuleItem
{
public:
    RuleItem(XSLElement *,double, QName &, QName *);
    ~RuleItem();
    XSLElement *rule;
    double priority;
    QName name,
        *mode;
};


/*****************************************************************

    R u l e S L i s t
    sorted list of rules

*****************************************************************/

class RuleSList : public SList<RuleItem*>
{
public:
    RuleSList();
    ~RuleSList();
    virtual int compare(int i, int j);
    XSLElement *findByName(QName &);
};

/*****************************************************************

DataLinesList

*****************************************************************/

class DataLineItem
{
public:
    ~DataLineItem();
    DataLine *_dataline;
    Tree *_tree;
    Bool _isXSL;
};

class DataLinesList: public PList<DataLineItem*>
{
public:
    int DataLinesList::findNum(Str &absoluteURI, Bool _isXSL,
        DLAccessMode _mode);
    Tree *getTree(Str &absoluteURI, Bool _isXSL, DLAccessMode _mode);
    eFlag addLine(DataLine *d, Tree *t, Bool isXSL);
};


/*****************************************************************

    P   r   o   c   e   s   s   o   r

*****************************************************************/

class VarsList;
class VertexList;

class Processor /* : public SabObj */
{
public:
    Tree *input,
        *styleSheet,
        *result;
    RuleSList rules;
    Processor();
    ~Processor();
    eFlag open(char *sheetURI, char *inputURI, char *resultURI);
    eFlag execute(Vertex *, Context *);
    eFlag execute(VertexList&, Context *);
    eFlag parse(Tree *, DataLine *d);
    double defaultPriorityLP(Expression *);
    double defaultPriority(XSLElement *);
    eFlag bestRule(XSLElement *&, Context *);
    eFlag builtinRule(Context *c);
    eFlag insertRule(XSLElement *);
    FILE *logfile;
    unsigned long allocObjects,
        allocBytes,
        maxAllocObjects,
        maxAllocBytes;
    Expression* getVarBinding(QName &);
    Bool cmpQNames(QName&, QName&);
    StrStrList aliases;
    Str& aliasXL(Str&);
    VarsList *vars;
    Tree *getOutTree();
    void setOutTree(Tree *);
    QName *getCurrentMode();
    void pushMode(QName *);
    void popMode();
    eFlag readTreeFromURI(Tree*& newTree, Str& location, Str& base, Bool isXSL);
    eFlag makeTreeForURI(Tree*& newTree, Str& location, Str& base);
    eFlag processVertexAfterParse(Vertex *, Tree *);
    SchemeHandler *getSchemeHandler();

    eFlag addLineParse(Tree *& newTree, Str &absolute, Bool isXSL);
    void copyArg(const Str& argName, int* argOrdinal,
       char*& newCopy);
    eFlag getArg(const char*, char*&, Bool);
    eFlag useArg(char *name, char *val);
    eFlag useGlobalParam(char *name, char *val);
    const char *hardBaseURI;
private:
    eFlag openCommon();
    Tree *outTree;
    eFlag execApply(Context *c);
    PList<QName*> modes;
    StrStrList
        argList;
    DataLinesList datalines;
};

#endif //ifndef ProcHIncl
