/****************************************
*                                       *
	s a b c m d
*                                       *
****************************************/

/****************************************
*                                       *
	i n c l u d e s
*                                       *
****************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../sablot.h"

/****************************************
*                                       *
	m a i n
*                                       *
****************************************/

#define ARG_OR_PAR_MAX 32

char usage[] =
	"SABCMD 0.33  (Apr 19, 2000)\n"\
    "Usage: sabcmd <stylesheet> [<input> [<output>]] [assignments]\n"\
    "By default, input = stdin, output = stdout\n"\
    "Assignments define named buffers (name=value) "\
    "and top-level params ($name=value)\n";

void saberr(char *msg1, char *msg2)
{
    fprintf(stderr, "Error: %s", msg1);
    if (msg2) fputs(msg2, stderr);
    fprintf(stderr, "\n\n%s", usage);
    exit(EXIT_FAILURE);
}

void copyAssignment(char **array, int index, char *text, int split)
{
    char *p;
    if (index >= ARG_OR_PAR_MAX)
      saberr("too many assignments", NULL);
    array[2 * index + 1] = text + split + 1;
    p = array[2 * index] = (char*) malloc(split + 1);
    memcpy(p, text, split);
    p[split] = 0;
}

int chrpos(char *text, char c)
{
    char *p = strchr(text, c);
    return p ? (int)(p - text) : -1;
}

void freefirst(char **array)
{
    for (; *array; array += 2)
        free(*array);
}

int main(int argc, char *argv[])
{
    int firstEq, i, pos,
        paramsI = 0, argsI = 0;
    char *paramsToPass[2 * ARG_OR_PAR_MAX  + 2],
        *argsToPass[2 * ARG_OR_PAR_MAX  + 2],
        *resultArg;

    for (i = 1; i < argc && (chrpos(argv[i], '=') == -1); i++);
    if (i == 1)
        saberr("stylesheet not given",NULL);
    if (i > 4)
        saberr("assignments must contain '=': ", argv[4]);
    firstEq = i;
    for ( ; i < argc; i++)
    {
        pos = chrpos(argv[i], '=');
        if (pos == -1)
            saberr("'=' expected: ",argv[i]);
        if (argv[i][0] == '$')
            copyAssignment(paramsToPass, paramsI++, argv[i] + 1, pos-1);
        else
            copyAssignment(argsToPass, argsI++, argv[i], pos);
    };

    argsToPass[2 * argsI] = NULL;
    paramsToPass[2 * paramsI] = NULL;

    if (SablotProcess(argv[1],
	    firstEq <= 2 ? (char*)"file://stdin" : argv[2],
		firstEq <= 3 ? (char*)"file://stdout" : argv[3],
		paramsToPass, argsToPass, &resultArg))
        return(1);
    if (resultArg)
    {
        fprintf(stderr, "output buffer sent to stdout\n");
        puts(resultArg);
        SablotFree(resultArg);
    }

    freefirst(argsToPass);
    freefirst(paramsToPass);

    return 0;
}





