/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef ShandlerHIncl
#define ShandlerHIncl

enum SchemeHandlerErrors
{
    SH_ERR_OK = 0,
    SH_ERR_NOT_OK = 1,
    SH_ERR_UNSUPPORTED_SCHEME,
};

/*****************************************************************
SchemeHandler 

  is a structure for a scheme handler. It contains pointers to
  the following functions of the handler: 
        open(), get(), put(), close().
  All of these function return an error flag (0=OK, 1=not). 
  open() may also return SH_ERR_UNSUPPORTED_SCHEME.
*****************************************************************/

/*  open: open the URI and return a handle
        scheme = URI scheme (e.g. "http")
        rest = the rest of the URI (without colon)
        the resulting handle is returned in '*handle'
*/
typedef int SchemeHandlerOpen(const char *scheme, const char *rest, int *handle);

/*  get: retrieve data from the URI
        handle = the handle assigned on open
        buffer = pointer to the data
        *byteCount = number of bytes to read 
            (the number actually read is returned here)
*/
typedef int SchemeHandlerGet(int handle, char *buffer, int *byteCount);

/*  put: save data to the URI (if possible)
        handle = the handle assigned on open
        buffer = pointer to the data
        *byteCount = number of bytes to write
            (the number actually written is returned here)
*/
typedef int SchemeHandlerPut(int handle, const char *buffer, int *byteCount);

/*  close: close the URI with the given handle
        handle = the handle assigned on open
*/
typedef int SchemeHandlerClose(int handle);

typedef struct
{
    SchemeHandlerOpen *open;
    SchemeHandlerGet *get;
    SchemeHandlerPut *put;
    SchemeHandlerClose *close;
} SchemeHandler;

#endif


