/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef SituaHIncl
#define SituaHIncl

#include "base.h"
#include "error.h"

//enum ErrCode;

/*
Mode constants for the situation object.
*/

enum SIT_MODE
{
    SIT_MODE_NONE
};

enum MsgType
{
    MT_ERROR,
        MT_WARN,
        MT_LOG
};

enum MsgDestination
{
    MD_NONE,
        MD_FILE,
        MD_CHERROR
};

/*
The following class keeps track of the current situation: which vertex
is currently being parsed/processed etc. This is used solely for error reporting.
*/

class XSLElement;
class Vertex;
#include "datastr.h"

class SitData 
{
public:
    Vertex *currV;
    Str currFile;
    int currLine;
};

class SituationObj
{
public:
    SituationObj();
    ~SituationObj();
    void setCurrV(Vertex *);
    void setCurrFile(Str &);
    void setCurrLine(int);
    //
    void error(MsgCode, const Str &, const Str &);
    void warning(MsgCode, const Str &, const Str &);
    void logmsg(MsgCode, const Str &, const Str &);
    //
    void clear();
    Bool isError();     
    //
    void msgOutputChError();
    eFlag msgOutputFile(char *_errwfn, char *_logfn);
    eFlag eraseLog(char *newLogFile);
    Str timeStr();
    eFlag closeFiles();
    void pushCurrV(Vertex *);
    void pushCurrent();
    void popCurrent();
private:
    Vertex *currV;
    Str currFile;
    int currLine;
    MsgCode pending;
    void message(MsgType, MsgCode, const Str &, const Str &);
    Bool useChError;
    FILE *logfile, *errwfile;
    PList<SitData*> history;
};

extern SituationObj situation;

#endif // SituaHIncl
