/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "tree.h"
#include "proc.h"

/****************************************
*                                       *
T r e e
*                                       *
****************************************/

/****************************************
T r e e   methods
****************************************/

Tree::Tree(const Str &aname, BOOL aXSLTree)
{
    name = aname;
    XSLTree = aXSLTree;
    stackTop = &root;
    pendingText = NULL;
    root.stamp = 0;
    vcount = 1;
    stripped = 0;
    dline = NULL;
};

Tree::~Tree()
{
}
    
eFlag Tree::appendVertex(Vertex *v)
{
    assert(stackTop && isDaddy(stackTop));
    E( cast(Daddy*,stackTop) -> newChild(v) ); //sets parent too
    if (isDaddy(v))
        stackTop = v;
    if (isText(v))
        pendingText = cast(Text*,v);
    else
        pendingText = NULL;
    if (isElement(v))
        toE(v) -> ownerT = this;
    v -> stamp = vcount++;
    return OK;
};

void Tree::dropCurrentElement(Vertex *v)
{
    assert(stackTop && isElement(stackTop));
    assert(stackTop == v);
    assert(!pendingText);
    stackTop = v -> parent;
    delete v;
    toE(stackTop) -> contents.deppend();
}

Vertex *Tree::appendText(char *s, int len)
{
    Vertex *txt;
    if (pendingText)
    {
        pendingText -> cont.nadd(s,len);
        return NULL;
    }
    else
    {
        appendVertex(txt = new Text(s,len));
        return txt;
    }
}

Vertex *Tree::popVertex()
{
    Vertex *v = NZ( stackTop );
    stackTop = v -> parent;
    pendingText = NULL;
    return v;
}

void Tree::output()
{
    root.output(*dline);
}

void Tree::setName(const Str& _name)
{
    name = _name;
}

