/*
 * Sablotron - XSL processor
 * Copyright (C) 2000 Ginger Alliance Ltd.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "verts.h"
#include "tree.h"
#include "expr.h"
#include "proc.h"
#include "context.h"
#include "vars.h"

/*****************************************************************
    Instruction tables
*****************************************************************/

struct AttTableItem
{
    XSL_ATT
        attCode;
    Bool 
        required,
        avtemplate;
    ExType
        exprType;
};

typedef AttTableItem AttTable[];

AttTable at_empty = 
{
    {XSLA_NONE, FALSE, FALSE, EX_NONE}
},
at_apply_templates  = 
{
    {XSLA_SELECT, FALSE, FALSE, EX_NODESET},
    {XSLA_MODE, FALSE, FALSE, EX_NONE}
},
at_attribute = 
{
    {XSLA_NAME, TRUE, TRUE, EX_STRING},
    {XSLA_NAMESPACE, FALSE, TRUE, EX_STRING}
},
at_attribute_set =
{
    {XSLA_NAME, TRUE, FALSE, EX_NONE},
    {XSLA_USE_ATTR_SETS, FALSE, FALSE, EX_NONE},
},
at_call_template = 
    {
        {XSLA_NAME, TRUE, FALSE, EX_NONE}
    }, 
        at_copy =
    {
        {XSLA_USE_ATTR_SETS, FALSE,  FALSE, EX_NONE}
    },
        at_copy_of =
    {
        {XSLA_SELECT, TRUE, FALSE, EX_UNKNOWN}
    },
        at_decimal_format =
    {
        {XSLA_NAME, FALSE, FALSE, EX_NONE},
        {XSLA_DECIMAL_SEPARATOR, FALSE, FALSE, EX_NONE},
        {XSLA_GROUPING_SEPARATOR, FALSE, FALSE, EX_NONE},
        {XSLA_INFINITY, FALSE, FALSE, EX_NONE},
        {XSLA_MINUS_SIGN, FALSE, FALSE, EX_NONE},
        {XSLA_NAN, FALSE, FALSE, EX_NONE},
        {XSLA_PERCENT, FALSE, FALSE, EX_NONE},
        {XSLA_PER_MILLE, FALSE, FALSE, EX_NONE},
        {XSLA_ZERO_DIGIT, FALSE, FALSE, EX_NONE},
        {XSLA_DIGIT, FALSE, FALSE, EX_NONE},
        {XSLA_PATTERN_SEPARATOR, FALSE, FALSE, EX_NONE}
    },
        at_element =
    {
        {XSLA_NAME, TRUE, TRUE, EX_STRING},
        {XSLA_NAMESPACE, FALSE, TRUE, EX_STRING},
        {XSLA_USE_ATTR_SETS, FALSE, FALSE, EX_NONE}
    },
        at_for_each =
    {
        {XSLA_SELECT, TRUE, FALSE, EX_NODESET}
    },
        at_if =
    {
        {XSLA_TEST, TRUE, FALSE, EX_BOOLEAN}
    },
        at_import =
    {
        {XSLA_HREF, TRUE, FALSE, EX_NONE}
    },
        at_include =
    {
        {XSLA_HREF, TRUE, FALSE, EX_NONE}
    },
        at_key =
    {
        {XSLA_NAME, TRUE, FALSE, EX_NONE},
        {XSLA_MATCH, TRUE, FALSE, EX_NODESET_PATTERN},
        {XSLA_USE, TRUE, FALSE, EX_UNKNOWN}
    },
        at_message =
    {
        {XSLA_TERMINATE, FALSE, FALSE, EX_NONE}
    },
        at_ns_alias =
    {
        {XSLA_STYLESHEET_PREFIX, TRUE, FALSE, EX_NONE},
        {XSLA_RESULT_PREFIX, TRUE, FALSE, EX_NONE}
    },
        at_number =
    {
        {XSLA_LEVEL, FALSE, FALSE, EX_NONE},
        {XSLA_COUNT, FALSE, FALSE, EX_NONE},
        {XSLA_FROM, FALSE, FALSE, EX_NONE},
        {XSLA_VALUE, FALSE, FALSE, EX_NONE},
        {XSLA_FORMAT, FALSE, TRUE, EX_STRING},
        {XSLA_LANG, FALSE, TRUE, EX_STRING},
        {XSLA_LETTER_VALUE, FALSE, TRUE, EX_STRING},
        {XSLA_GROUPING_SEPARATOR, FALSE, TRUE, EX_STRING},
        {XSLA_GROUPING_SIZE, FALSE, TRUE, EX_STRING}
    },
        at_output =
    {
        {XSLA_METHOD, FALSE, FALSE, EX_NONE},
        {XSLA_VERSION, FALSE, FALSE, EX_NONE},
        {XSLA_ENCODING, FALSE, FALSE, EX_NONE},
        {XSLA_OMIT_XML_DECL, FALSE, FALSE, EX_NONE},
        {XSLA_STANDALONE, FALSE, FALSE, EX_NONE},
        {XSLA_DOCTYPE_PUBLIC, FALSE, FALSE, EX_NONE},
        {XSLA_DOCTYPE_SYSTEM, FALSE, FALSE, EX_NONE},
        {XSLA_CDATA_SECT_ELEMS, FALSE, FALSE, EX_NONE},
        {XSLA_INDENT, FALSE, FALSE, EX_NONE},
        {XSLA_MEDIA_TYPE, FALSE, FALSE, EX_NONE}
    },
        at_param =
    {
        {XSLA_NAME, TRUE, FALSE, EX_NONE},
        {XSLA_SELECT, FALSE, FALSE, EX_UNKNOWN}
    },
        at_preserve_space =
    {
        {XSLA_ELEMENTS, TRUE, FALSE, EX_NONE}
    },
        at_pi =
    {
        {XSLA_NAME, TRUE, TRUE, EX_STRING}
    },
        at_sort =
    {
        {XSLA_SELECT, FALSE, FALSE, EX_STRING},
        {XSLA_LANG, FALSE, TRUE, EX_STRING},
        {XSLA_DATA_TYPE, FALSE, TRUE, EX_STRING},
        {XSLA_ORDER, FALSE, TRUE, EX_STRING},
        {XSLA_CASE_ORDER, FALSE, TRUE, EX_STRING}
    },
        at_strip_space =
    {
        {XSLA_ELEMENTS, TRUE, FALSE, EX_NONE}
    },
        at_template =
    {
        {XSLA_MATCH, FALSE, FALSE, EX_NODESET_PATTERN},
        {XSLA_NAME, FALSE, FALSE, EX_NONE},
        {XSLA_PRIORITY, FALSE, FALSE, EX_NONE},
        {XSLA_MODE, FALSE, FALSE, EX_NONE}
    },
        at_text =
    {
        {XSLA_DISABLE_OUTPUT_ESC, FALSE, FALSE, EX_NONE}
    },
        at_transform =
    {
        {XSLA_ID, FALSE, FALSE, EX_NONE},
        {XSLA_EXT_ELEM_PREFIXES, FALSE, FALSE, EX_NONE},
        {XSLA_EXCL_ELEM_PREFIXES, FALSE, FALSE, EX_NONE},
        {XSLA_VERSION, TRUE, FALSE, EX_NONE}
    },
        at_value_of =
    {
        {XSLA_SELECT, TRUE, FALSE, EX_STRING},
        {XSLA_DISABLE_OUTPUT_ESC, FALSE, FALSE, EX_NONE}
    },
        at_variable =
    {
        {XSLA_NAME, TRUE, FALSE, EX_NONE},
        {XSLA_SELECT, FALSE, FALSE, EX_UNKNOWN}
    },
        at_when =
    {
        {XSLA_TEST, TRUE, FALSE, EX_BOOLEAN}
    },
        at_with_param =
    {
        {XSLA_NAME, TRUE, FALSE, EX_NONE},
        {XSLA_SELECT, FALSE, FALSE, EX_UNKNOWN}
    };

#define instr(code, elemflg, req, max, table) \
{code, elemflg, req, max, (AttTableItem*) table}

enum ElemFlags
{
//    ELEM_EMPTY = 1,
    ELEM_TOPLEVEL = 2,
    ELEM_INSTR = 4,
    ELEM_EXTRA = 8,
    //
    ELEM_CONT_PCDATA = 0x10,
    ELEM_CONT_TOPLEVEL = 0x20,
    ELEM_CONT_INSTR = 0x40,
    ELEM_CONT_EXTRA = 0x80,
    //
    ELEM_SELF = ELEM_TOPLEVEL | ELEM_INSTR | ELEM_EXTRA,
    ELEM_CONT = ELEM_CONT_TOPLEVEL | ELEM_CONT_INSTR | ELEM_CONT_EXTRA,
    //
    ELEM_I0 = ELEM_INSTR,
    ELEM_IX = ELEM_INSTR | ELEM_CONT_EXTRA,
    ELEM_II = ELEM_INSTR | ELEM_CONT_INSTR,
    ELEM_IT_I = ELEM_INSTR | ELEM_TOPLEVEL | ELEM_CONT_INSTR,
    ELEM_I_XI = ELEM_INSTR | ELEM_CONT_EXTRA | ELEM_CONT_INSTR,
    ELEM_I__ = ELEM_INSTR | ELEM_CONT_PCDATA,
    //
    ELEM_T0 = ELEM_TOPLEVEL,
    ELEM_TX = ELEM_TOPLEVEL | ELEM_CONT_EXTRA,
    ELEM_TX_I = ELEM_TOPLEVEL | ELEM_EXTRA | ELEM_CONT_INSTR,
    ELEM_T_XI = ELEM_TOPLEVEL | ELEM_CONT_EXTRA | ELEM_CONT_INSTR,
    //
    ELEM_X0 = ELEM_EXTRA,
    ELEM_XI = ELEM_EXTRA | ELEM_CONT_INSTR,
    ELEM_X_XT = ELEM_EXTRA | ELEM_CONT_EXTRA | ELEM_CONT_TOPLEVEL,
    ELEM_NONE
};

struct InstrTableItem
{
    XSL_OP op;
    ElemFlags flags;
    int 
        reqAtts,
        maxAtts;
    AttTableItem *att;
} instrTable[]
=
{
    instr(XSL_APPLY_IMPORTS,    ELEM_I0,    0, 0, at_empty),
    instr(XSL_APPLY_TEMPLATES,  ELEM_IX,    0, 2, at_apply_templates),
    instr(XSL_ATTRIBUTE,        ELEM_II,    1, 2, at_attribute),
    instr(XSL_ATTRIBUTE_SET,    ELEM_TX,    1, 2, at_attribute_set),
    instr(XSL_CALL_TEMPLATE,    ELEM_IX,    1, 1, at_call_template),
    instr(XSL_CHOOSE,           ELEM_IX,    0, 0, at_empty),
    instr(XSL_COMMENT,          ELEM_II,    0, 0, at_empty),
    instr(XSL_COPY,             ELEM_II,    0, 1, at_copy),
    instr(XSL_COPY_OF,          ELEM_I0,    1, 1, at_copy_of),
    instr(XSL_DECIMAL_FORMAT,   ELEM_T0,    0, 11, at_decimal_format),
    instr(XSL_ELEMENT,          ELEM_II,    1, 3, at_element),
    instr(XSL_FALLBACK,         ELEM_II,    0, 0, at_empty),
    instr(XSL_FOR_EACH,         ELEM_I_XI,  1, 1, at_for_each),
    instr(XSL_IF,               ELEM_II,    1, 1, at_if),
    instr(XSL_IMPORT,           ELEM_X0,    1, 1, at_import),
    instr(XSL_INCLUDE,          ELEM_T0,    1, 1, at_include),
    instr(XSL_KEY,              ELEM_T0,    3, 3, at_key),
    instr(XSL_MESSAGE,          ELEM_II,    0, 1, at_message),
    instr(XSL_NAMESPACE_ALIAS,  ELEM_T0,    2, 2, at_ns_alias),
    instr(XSL_NUMBER,           ELEM_I0,    0, 9, at_number),
    instr(XSL_OTHERWISE,        ELEM_XI,    0, 0, at_empty),
    instr(XSL_OUTPUT,           ELEM_T0,    0, 10, at_output),
    instr(XSL_PARAM,            ELEM_TX_I,  1, 2, at_param),
    instr(XSL_PRESERVE_SPACE,   ELEM_T0,    1, 1, at_preserve_space),
    instr(XSL_PROCESSING_INSTR, ELEM_II,    1, 1, at_pi),
    instr(XSL_SORT,             ELEM_X0,    0, 5, at_sort),
    instr(XSL_STRIP_SPACE,      ELEM_T0,    1, 1, at_strip_space),
    instr(XSL_STYLESHEET,       ELEM_X_XT,  1, 4, at_transform),
    instr(XSL_TEMPLATE,         ELEM_T_XI,  0, 4, at_template),
    instr(XSL_TEXT,             ELEM_I__,   0, 1, at_text),
    instr(XSL_TRANSFORM,        ELEM_X_XT,  1, 4, at_transform),
    instr(XSL_VALUE_OF,         ELEM_I0,    1, 2, at_value_of),
    instr(XSL_VARIABLE,         ELEM_IT_I,  1, 2, at_variable),
    instr(XSL_WHEN,             ELEM_XI,    1, 1, at_when),
    instr(XSL_WITH_PARAM,       ELEM_XI,    1, 2, at_with_param)
};

/****************************************
V e r t e x
****************************************/

Vertex::~Vertex()
{
};

eFlag Vertex::execute(Context *c)
{
    assert(FALSE);
    return NOT_OK;
}

eFlag Vertex::value(Str& ret, Context *c)
{
    assert(0);
    return NOT_OK;
}

void Vertex::copy(Tree *destTree, Bool full)
{
    assert(0);
}

void Vertex::speak(Str &ret, SpeakMode mode)
{
}

void Vertex::output(DataLine& outbox)
{
    Str s;
    speak(s, (SpeakMode)(SM_NAME | SM_CONTENTS));
    if (!s.isEmpty())
        outbox.save(s);
}

void Vertex::setParent(Vertex *v)
{
    parent = v;
}

const QName& Vertex::getName() const
{
    return name;
}


/****************************************
V e r t e x L i s t
****************************************/

VertexList::VertexList()
: PList<Vertex*>()
{
}

VertexList::~VertexList()
{
    deppendall();
}

eFlag VertexList::execute(Context *c)
{
    int i;
    for (i=0; i<number(); i++)
        E( (*this)[i] -> execute(c) );
    return OK;
}

eFlag VertexList::value(Str &ret, Context *c)
{
    int i;
    Str temp;

    ret.empty();
    for (i=0; i<number(); i++)
    {
        E( (*this)[i] -> value(temp, c) );
        ret.add(temp);
        temp.remove();
    };
    return OK;
}

void VertexList::speak(Str &ret, SpeakMode mode)
{
    int i;
    for (i = 0; i < number(); i++)
    {
        (*this)[i] -> speak(ret, mode);
        if ((mode & SM_INS_SPACES) && (i < number() - 1))
             ret += ' ';
    };
}

void VertexList::append(Vertex *v)
{
    v -> ordinal = number();
    List<Vertex*>::append(v);
}

int VertexList::strip()
{
    int cnt = 0;
    for (int i = 0; i < number(); i++)
    {
        if (((*this)[i] -> vt == VT_TEXT) && 
            cast(Text*,(*this)[i]) -> allWhite())
        {
            cnt++;
            freerm(i--,FALSE);
        }
    };
    return cnt;
}

void VertexList::copy(Tree *destTree, Bool full)
{
    for (int i = 0; i < number(); i++)
        (*this)[i] -> copy(destTree, full);
}


/****************************************
D a d d y
a vertex with contents
****************************************/

Daddy::~Daddy()
{
    contents.freeall(FALSE);
};

eFlag Daddy::execute(Context *c)
{
    situation.pushCurrV(this);
    E( contents.execute(c) );
    situation.popCurrent();
    return OK;
};

eFlag Daddy::value(Str &ret, Context *c)
{
    E( contents.value(ret, c) );
    return OK;
}

eFlag Daddy::checkChildren()
{
    assert(0);
    return NOT_OK;
}

eFlag Daddy::newChild(Vertex *v)
{
    contents.append(v);
    v -> setParent(this);
    return OK;
};

void Daddy::speak(Str &ret, SpeakMode mode)
{
    if (mode && SM_CONTENTS)
        contents.speak(ret, mode);
}

int Daddy::strip()
{
    return contents.strip();
}

/****************************************
R o o t N o d e
a vertex with prolog and contents 
****************************************/

RootNode::~RootNode()
{
}

eFlag RootNode::execute(Context *c)
{
    //! process prolog
    //
    E( Daddy::execute(c) );
    return OK;
};

eFlag RootNode::checkChildren()
{
    return OK;
}

eFlag RootNode::newChild(Vertex *v)
{
    //! insert prolog
    //
    E( Daddy::newChild(v) );
    return OK;
};

void RootNode::speak(Str &s, SpeakMode mode)
{
    if (mode & SM_DESCRIBE)
        s += "[ROOT]";
    if (mode & SM_CONTENTS)
        Daddy::speak(s,mode);
}

void RootNode::copy(Tree *destTree, Bool full)
{
    contents.copy(destTree, full);
    /*
    for (int i = 0; i < contents.number(); i++)
        contents[i] -> copy(destTree, full);
    */
}

/****************************************
E l e m e n t
a vertex with attributes, defs and contents
****************************************/

Element::Element(QName& aqname, Tree *_ownerT, VTYPE avt /*= VT_ELEMENT_WF*/)
: Daddy(avt)
{
    name = aqname;
    ownerT = _ownerT;
};

Element::~Element()
{
    namespaces.freeall(FALSE);
    atts.freeall(FALSE);
};

eFlag Element::execute(Context *c)
{
    Tree *out = proc -> getOutTree();
    out -> appendVertex(new Element(name,out));
    atts.execute(c);
    E( Daddy::execute(c) );
    proc -> getOutTree() -> popVertex();
    removeBindings();
    return OK;
}

eFlag Element::newChild(Vertex *v)
{
    v -> setParent(this);
    if isAttr(v)
        atts.append(v);
    else
    {
        if (isNS(v))
            namespaces.append(v);
        else
            E( Daddy::newChild(v) );
    }
    return OK;
};

void Element::speak(Str &ret, SpeakMode mode)
{
    if (mode & (SM_NAME | SM_CONTENTS))
    {
        ret += '<';
        name.speak(ret,mode);
        if (mode & SM_CONTENTS)
        {
            if (atts.number())
            {
                ret += ' ';
                atts.speak(ret, (SpeakMode)(mode | SM_INS_SPACES));
            };
/*
            commented this out as the closed empty tags
            confuse poor Navigator:

            if (!contents.number())
                ret += "/>";
            else
*/
            {
                ret += '>';
                contents.speak(ret,(SpeakMode)(mode & ~SM_INS_SPACES));
                //        ret += Str("</"); this caused errors!
                ret += "</";
                name.speak(ret,mode);
                ret += '>';
            };
        }
        else ret += '>';
    };
};

/*================================================================
removeBindings
    This removes all VARIABLE bindings for the variables
    that are children of this element. The param and with-param
    bindings are removed by the 'vars' list itself.
================================================================*/

void Element::removeBindings()
{
    Attribute *a;
    Vertex *v;
    for (int i = contents.number() - 1; i >= 0; i--)
    {
        v = contents[i];
        if (isXSLElement(v) && (cast(XSLElement*, v) -> op == XSL_VARIABLE))
        {
            QName q;
            a = NZ(cast(XSLElement*, v) -> atts.find(XSLA_NAME));
            q.setLogical(a -> cont, &namespaces, FALSE);
            proc -> vars -> rmBinding(q);
        }
    }
}

void Element::copy(Tree *destTree, Bool full)
{
//    int i;
    destTree -> appendVertex(
        isXSLElement(this)?
            new XSLElement(name, destTree, toX(this) -> op) :
            new Element(name, destTree)) ;
    if (isXSLElement(this))
    namespaces.copy(destTree, full);
    atts.copy(destTree,full);
    contents.copy(destTree,full);
/*
    for (i = 0; i < namespaces.number(); i++)
        namespaces[i] -> copy(destTree, full);
    for (i = 0; i < atts.number(); i++)
        atts[i] -> copy(destTree, full);
    for (i = 0; i < contents.number(); i++)
        contents[i] -> copy(destTree, full);
*/
    destTree -> popVertex();
}

/****************************************
A t t r i b u t e
****************************************/

Attribute::Attribute(const QName &aqname, const Str &acont, 
                     XSL_ATT aop)
: Vertex((VTYPE) (VT_ATTRIBUTE_WF | 
    (aop != XSLA_NONE ? VT_XSL : 0)))
{
    expr = NULL;
    name = aqname;
    cont = acont;
    op = aop;
};

Attribute::~Attribute()
{
    if (expr)
        delete expr;
}

eFlag findBrace(char *&p, char which, Str &copybuf)
{
    char *p0 = p,
        c;
    int len;
    copybuf.empty();
    while (*p)
    {
        if (((c = *p) == '"') || (c == '\''))
        {
            do p++; while (*p && (*p != c));
            if (!*p) Err(ET_INFINITE_LITERAL);
        };
        if (*p != which) 
            p++;
        else
        {
            if (*(p+1) == which)
            {
                len = p+1 - p0;
                if (len)
                    copybuf.nadd(p0, len);
                p += 2;
                p0 = p;
            }
            else break;
        };
    };
    len = p - p0;
    if (len) copybuf.nadd(p0, len);
    return OK;
}

eFlag Attribute::buildExpr(Bool asTemplate, ExType ty)
{
    situation.pushCurrV(this);
    char *q;
    Expression *eadd;
    if (asTemplate)
    {
        Str sadd;
        expr = new Expression(toE(parent),EXF_STRINGSEQ);
        char *p = (char*) cont;
        while (*p)
        {
            E( findBrace(q = p, '{', sadd) );
            if (!sadd.isEmpty())
            {
                eadd = new Expression(toE(parent),EXF_ATOM);
                eadd -> setAtom(sadd);
                expr -> args.append(eadd);
            };
            if (!*q) break;
            //
            if (!*(p = q + 1)) break;
            E( findBrace(q = p, '}',sadd) );
            if (!sadd.isEmpty())
            {
                eadd = new Expression(toE(parent));
                E( eadd -> parse(sadd));
                expr -> args.append(eadd);
            };
            if (!*q) break;
            p = q + 1;
        }
    }
    else
    {
        expr = new Expression(toE(parent));
        E( expr -> parse(cont, (Bool) (ty == EX_NODESET_PATTERN)) );
    }
    situation.popCurrent();
    return OK;
}

eFlag Attribute::execute(Context *c)
{
    assert(parent);
    if (isXSLElement(parent))
        return OK;
    situation.pushCurrV(this);
    Str temp;
    E( value(temp,c) );
    proc -> getOutTree() -> appendVertex(
        new Attribute(name,temp,XSLA_NONE));
    situation.popCurrent();
    return OK;
}

eFlag Attribute::value(Str &ret, Context *c)
{
    if (expr)
    {
        Expression temp(toE(parent));
        E( expr -> eval(temp, c) );
        ret = temp.tostring();
    }
    else
        ret = cont;
    return OK;
}

void Attribute::speak(Str &ret, SpeakMode mode)
{
    if (mode & (SM_NAME | SM_CONTENTS))
        name.speak(ret,mode);
    if (mode & SM_CONTENTS)
    {
        ret += "=\"";
        ret += cont;
        ret += "\"";
    }
}

void Attribute::copy(Tree *destTree, Bool full)
{
    destTree -> appendVertex(
        new Attribute(name,cont,op));
}

/****************************************
A t t L i s t
****************************************/

Attribute *AttList::find(XSL_ATT what)
{
    int i;
    Attribute *a;
    for (i = 0; i < number(); i++)
    {
        a = cast(Attribute *, (*this)[i]);
        if (a -> op == what)
            return a;
    };
    return NULL;
}

/*****************************************************************
    N m S p a c e
*****************************************************************/

NmSpace::NmSpace(const Str &_prefix, const Str &_uri)
: Vertex(VT_NAMESPACE)
{
    prefix = _prefix;
    uri = _uri;
}

NmSpace::~NmSpace()
{
}

eFlag NmSpace::execute(Context *c)
{
    assert(parent);
    if (isXSLElement(parent))
        return OK;
    proc -> getOutTree() -> appendVertex(
        new NmSpace(proc -> aliasXL(prefix),uri));
    return OK;
}

void NmSpace::speak(Str& s, SpeakMode mode)
{
    if (!prefix.isEmpty())
        s.add(prefix + ":");
    s += "xmlns=\"";
    s += uri;
    s += '\"';
}

eFlag NmSpace::value(Str &s, Context *c)
{
    s = uri;
    return OK;
}

void NmSpace::copy(Tree *destTree, Bool full)
{
    destTree -> appendVertex(
        new NmSpace(prefix,uri));
}

/*****************************************************************
    N S L i s t
*****************************************************************/

NSList::~NSList()
{
    freeall(FALSE);
}

NmSpace *NSList::find(Str &prefix) const
{
    NmSpace *nsitem;
    for (int i = 0; i < number(); i++)
        if ((nsitem = toNS((*this)[i])) -> prefix 
            == prefix)
            return nsitem;
    return NULL;
}

/*
NSList& NSList::operator= (const NSList& other)
{
    freeall(FALSE);
    for (int i = 0; i < other.number(); i++)
        append(new NmSpace(*cast(NmSpace*,other[i])));
    return *this;
}
*/

eFlag NSList::resolve(Str &what, Bool defaultToo)
{
    Bool emptystr = what.isEmpty();
    if (emptystr && !defaultToo)
        return OK;
    NmSpace *p = find(what);
    if (!p)
    {
        if (emptystr)
            return OK;
        else
            Err1(ET_BAD_PREFIX, (char*) what);
    }
    else
    {
        what = p -> uri;
    }
    return OK;
}

void NSList::unresolve(Str &what) const
{
    assert(!what.isEmpty());
    NmSpace *currNS;
    for (int i = 0; i < number(); i++)
    {
        currNS = toNS((*this)[i]);
        if (what == currNS -> uri)
        {
            what = currNS -> prefix;
            return;
        }
    };
    assert(0);
}

/*
appends all current namespace declarations as vetices to tree t
'other' is the list where they are being appended in t
*/

void NSList::giveCurrent(NSList &other, Tree *t) const
{
    const NmSpace *currNS;
    for (int i = number() - 1; i >= 0; i--)
    {
        currNS = toNS((*this)[i]);
	const Str& currPfx = currNS -> prefix;
        if (!other.find((Str&) currPfx))
            t -> appendVertex(new NmSpace(currPfx, currNS -> uri));
            // other.append(new NmSpace(currPfx, currNS -> uri));
    }
}

/****************************************
T e x t
****************************************/

Text::~Text()
{
}

eFlag Text::execute(Context *c)
{
    proc -> getOutTree() -> appendVertex(new Text(cont));
    return OK;
}

eFlag Text::value(Str& ret, Context *c)
{
    ret = cont;
    return OK;
}

void Text::speak(Str &ret, SpeakMode mode)
{
    if (!(mode & SM_CONTENTS)) 
    {
        if (mode & SM_DESCRIBE)
            ret += "[TEXT]";
        return;
    };
    if (mode & SM_DESCRIBE)
        ret += "text \"";
    if (mode & SM_ESCAPE)
        cont.speakTerse(ret);
    else
        ret += cont;
    if (mode & SM_DESCRIBE)
        ret += "\"";
};

Bool Text::allWhite()
{
    for (char *p = (char*) cont; *p; p++)
    {
        switch(*p)
        {
        case 9:
        case 0xa:
        case 0xd:
        case ' ':
            break;
        default:
            return FALSE;
        };
    }
    return TRUE;
}

void Text::copy(Tree *destTree, Bool full)
{
    destTree -> appendVertex(
        new Text(cont));
}

/****************************************
X S L E l e m e n t
****************************************/

XSLElement::XSLElement(QName& aqname, Tree *_ownerT, XSL_OP code)
:
Element(aqname, _ownerT, VT_XSL_ELEMENT_WF)
{
    assert(code != XSL_NONE);
    op = code;
};


eFlag XSLElement::newChild(Vertex *v)
{
    // process defs
    //
    E( Element::newChild(v) );
    return OK;
};

eFlag XSLElement::execute(Context *c)
{
    Attribute *a;
    // ???
    if (c -> isFinished())
        return OK;

    situation.pushCurrV(this);
    Vertex *v = c -> current();
    assert(v);

    switch(op)
    {
    case XSL_APPLY_TEMPLATES:
        {
            Attribute *aSelect = atts.find(XSLA_SELECT);
            Expression *e;
            if (aSelect)
                e = NZ(aSelect -> expr);
            else
            {
                e = new Expression(this, EXF_LOCPATH);
                e -> setLS(
                    AXIS_CHILD, 
                    EXNODE_NODE);
            };

            // mode
            Bool addedMode = TRUE;
            a = atts.find(XSLA_MODE);
            if (a)
            {
                QName *m = new QName;
                E( m -> setLogical(a -> cont,&namespaces,FALSE) );
                proc -> pushMode(m);
            }
            else if (proc -> getCurrentMode())
                proc -> pushMode(NULL);
            else addedMode = FALSE;
            
            Context *newc = c;
            E( e -> createContext( newc ) ); 
            if (!newc -> isVoid())
            {
                // process with-params
                E( contents.execute(c) );
                E( proc -> execute((Vertex*) NULL, newc) );
                // remove the prebindings introduced by with-param
                proc -> vars -> rmPrebindings();
            }
            else
                delete newc;
            if (addedMode)
                proc -> popMode();
            if (!aSelect)
                delete e;
        }; break;
    case XSL_IF:
        {
            a = NZ( atts.find(XSLA_TEST) );
            Expression boolexpr(this);
            E( NZ(a -> expr) -> eval(boolexpr,c) );
            Bool boolval = boolexpr.tobool();
            if ((boolval) && (contents.number()))
//                E( proc -> execute(contents[0],c) );
                E( contents.execute(c) );
        }; break;
    case XSL_CHOOSE:
        {
            Bool done = FALSE;
            for (int i = 0; i < contents.number() && !done; i++)
            {
                XSLElement *x = cast(XSLElement*,contents[i]);
                if (x -> op == XSL_WHEN)
                {
                    E(  NZ(NZ(x -> atts.find(XSLA_TEST)) 
                        -> expr) -> trueFor(c,done)     );
                    if (done)
                        E( x -> execute(c) );
                }
                else
                {
                    assert(x -> op == XSL_OTHERWISE);
                    x -> execute(c);
                }
            };
        }; break;
    case XSL_WHEN: // condition tested by CHOOSE
        {
            E( contents.execute(c) );
        }; break;
    case XSL_OTHERWISE: // condition tested by CHOOSE
        {
            E( contents.execute(c) );
        }; break;
    case XSL_ELEMENT:
        {
            QName q;
            Str nameStr;
            E( NZ( atts.find(XSLA_NAME) ) -> value(nameStr, c) );
            E( q.setLogical(nameStr, &namespaces, TRUE) );
            proc -> getOutTree() -> appendVertex(new Element(q, ownerT));
            E( Daddy::execute(c) );
            proc -> getOutTree() -> popVertex(); 
        }; break;
/*
    case XSL_PROCESSING_INSTR:
        {
            QName q;
            Str nameStr;
            E( NZ( atts.find(XSLA_NAME) ) -> value(nameStr, c) );
            E( q.setLogical(nameStr,&namespaces,FALSE) );
            if ((!q.prefix -> isEmpty) || 
                (strEqNoCase(q.local,"xml")))
                Err1(E_PI_TARGET,(char*) nameStr);
            proc -> pushOutTree();
            E( contents.execute() );
            if (!treeIsTextOnly(proc -> getOutTree()))
                Err(E_PI_CONTENTS);
            proc -> popOutTree();
            proc -> getOutTree() -> appendVertex(new Element(q));
            E( Daddy::execute(c) );
            proc -> getOutTree() -> popVertex(); 
        }; break;
    case XSL_COMMENT:
        {
        }; break;
*/
    case XSL_ATTRIBUTE:
        {
          QName q;
          Str nameStr;
            E( NZ( atts.find(XSLA_NAME) ) -> value(nameStr, c) );
            E( q.setLogical(nameStr, &namespaces, FALSE) );
            Str cont;
            E( Daddy::value(cont, c));
            proc -> getOutTree() -> appendVertex(
                new Attribute(q, cont, XSLA_NONE));
        }; break;
    case XSL_TEXT:
        {
            Str cont;
            E( Daddy::value(cont, c));
            proc -> getOutTree() -> appendText(cont, cont.length());
        }; break;
    case XSL_TEMPLATE:
        {
            proc -> vars -> startApplyOne();
            E( Daddy::execute(c) );
            proc -> vars -> endApplyOne();
        }; break;
    case XSL_STYLESHEET:
    case XSL_TRANSFORM:
        {
            Context *newc = c -> copy();

            // process globals and other top level elements
            for (int i = 0; i < contents.number(); i++)
            {
                Vertex *son = contents[i];
                if (isXSLElement(son) && (instrTable[toX(son) -> op].flags & ELEM_TOPLEVEL))
                {
                    switch(toX(son) -> op)
                    {
                    case XSL_TEMPLATE: break;
                    default: E( son -> execute(c) );
                    }
                }
            }

            proc -> vars -> startCall();
            E( proc -> execute((Vertex*) NULL, newc ));
            // newc has been deleted by now
            proc -> vars -> endCall();
        }; break;
    case XSL_VALUE_OF:
        {
            a = NZ( atts.find(XSLA_SELECT) );
            Expression temp(this);
            E( NZ( a -> expr ) -> eval(temp, c) );
            Str cont = temp.tostring();
            proc -> getOutTree() -> appendVertex(new Text(cont));
        };
        break;
    case XSL_COPY_OF:
        {
            a = NZ( atts.find(XSLA_SELECT) );
            Expression temp(this);
            E( NZ( a -> expr ) -> eval(temp, c) );
            if (temp.type == EX_NODESET)
            {
                const Context& ctxt = temp.tonodesetRef();
                for (int k = 0; k < ctxt.number(); k++)
                    ctxt[k] -> copy(proc -> getOutTree(), TRUE);
            }
            else
            {
                Str cont = temp.tostring();
                proc -> getOutTree() -> appendVertex(new Text(cont));
            }
        }; break;
    case XSL_FOR_EACH:
        {
            a = NZ( atts.find(XSLA_SELECT) );
            Context *newc = c;
            E( NZ( a -> expr ) -> createContext(newc));
            if (!newc -> isVoid() && contents.number())
            {
                E( proc -> execute(contents, newc) );
                // E( proc -> execute(contents[0],newc) );
                // proc disposes of the new context
            }
            else
                delete newc;
        };
        break;
    case XSL_CALL_TEMPLATE:
        {
            QName q;
            a = NZ( atts.find(XSLA_NAME) );
            XSLElement *thatrule;
            E( q.setLogical(a -> cont, &namespaces, FALSE) );
            if (!(thatrule = proc -> rules.findByName(q)))
                Err(ET_RULE_NOT_FOUND);

//            proc -> vars -> rmPrebindings();
            //process with-param
            E( contents.execute(c) );
            //execute the other rule
//            proc -> vars -> startLevel();
            E( thatrule -> execute(c) );
            proc -> vars -> rmPrebindings();
//            proc -> vars -> endLevel();
        }; break;
    case XSL_VARIABLE:
    case XSL_WITH_PARAM:
    case XSL_PARAM:
        {
            QName q;

            Bool freeit = FALSE;
            Expression *expr = NULL;

            // there must be a 'name' attribute
            a = NZ(atts.find(XSLA_NAME));
            // stick the name into q referring to this element's namespace decl's
            q.setLogical(a -> cont, &namespaces, FALSE);

            // if there's a 'select', use the expression it gives
            a = atts.find(XSLA_SELECT);
            if (a)
                expr = NZ(a -> expr);

            // otherwise, construct a new expression; it may be an empty string
            // if this element has void content, or a result tree fragment otherwise 
            else
            {
                freeit = TRUE;
                expr = new Expression(this);
                if (contents.isEmpty())
                    expr -> setAtom(theEmptyString);
                else // result tree fragment
                {
                    // start to output to a new tree, remembering the old one
                    Tree *wasout = proc -> getOutTree();

                    // starting a nesting will make the current prebindings
                    // invisible
                    proc -> vars -> startNested();

                    // set the new output tree
                    proc -> setOutTree(expr -> setFragment());

                    // execute the inside to create the fragment
                    E( contents.execute(c) );

                    // end the shadowing of preexisting bindings
                    proc -> vars -> endNested();

                    // restore the original output tree
                    proc -> setOutTree(wasout);
                };
            }
            //
            // evaluate the expression
            Expression *temp = new Expression(this);
            E( expr -> eval(*temp,c) );
            temp -> pTree = expr -> pTree;
            expr -> pTree = NULL;

            // if the expression was newly constructed then drop it
            if (freeit)
                delete NZ(expr);

            // add the new binding
            switch(op)
            {
            case XSL_PARAM:
                {
                    E( proc -> vars -> addBinding(q, temp, FALSE) );
                }; break;
            case XSL_WITH_PARAM:
                {
                    E( proc -> vars -> addPrebinding(q, temp) );
                }; break;
            case XSL_VARIABLE:
                {
                    E( proc -> vars -> addBinding(q, temp, TRUE) );
                }; break;
            }
        }; break;
    default: 
        Err(ET_UNSUPP_XSL);
    };

    //remove the variable bindings that occured inside this element
    if ((op != XSL_TEMPLATE) && (op != XSL_TRANSFORM) && (op != XSL_STYLESHEET))
        removeBindings();
    situation.popCurrent();
    return OK;
}

Expression *XSLElement::getAttExpr(XSL_ATT code)
{
    Attribute *a = atts.find(code);
    return a ? (a -> expr) : NULL;
}

int XSLElement::strip()
{
    if (op != XSL_TEXT)
        return defs.strip() + contents.strip();
    else return 0;
}


/*................................................................
findAttNdx()
    returns the index of given attribute in the attribute table
    or -1 if not found
................................................................*/

int findAttNdx(InstrTableItem &iitem, Attribute *a)
{
    for (int i = 0; i < iitem.maxAtts; i++)
        if (iitem.att[i].attCode == a -> op) return i;
    return -1;
}

/*================================================================
checkAtts
    called when all attributes have been parsed in. Returns FALSE
    iff they are OK. Return value of TRUE means that the end tag handler
    signals error.
================================================================*/

eFlag XSLElement::checkAtts()
{
    InstrTableItem &instrData = instrTable[op];
    assert(instrData.op == op);
    int
        attNdx,
        reqCount = 0;
    Attribute *a;
    situation.pushCurrV(this);
    for (int i = 0; i < atts.number(); i++)
    {
        a = cast(Attribute*, atts[i]);
        if ((attNdx = findAttNdx(instrData,a)) == -1)
            Err1(ET_BAD_ATTR,a -> name.getname());
        if (instrData.att[attNdx].required)
            reqCount++;
        if (instrData.att[attNdx].exprType != EX_NONE)
            E( a -> buildExpr(
                instrData.att[attNdx].avtemplate,
                instrData.att[attNdx].exprType) );
    };
    if (reqCount < instrData.reqAtts)
        Err(ET_REQ_ATTR);
    situation.popCurrent();
    return OK;
};

//................................................................

void XSLElement::checkExtraChildren(int& k)
{
    Vertex *w;
    XSL_OP hisop;
    int status = 0;
    for (k = 0; k < contents.number(); k++)
    {
        w = contents[k];
        if (!isXSLElement(w)) return;
        hisop = toX(w) -> op;
        switch(op)
        {
        case XSL_APPLY_TEMPLATES:
            if ((hisop != XSL_SORT) && (hisop != XSL_WITH_PARAM)) return;
            break;
        case XSL_ATTRIBUTE_SET:
            if (hisop != XSL_ATTRIBUTE) return;
            break;
        case XSL_CALL_TEMPLATE:
            if (hisop != XSL_WITH_PARAM) return;
            break;
        case XSL_CHOOSE:
            switch(hisop)
            {
            case XSL_WHEN:
                {
                    if (status <= 1) status = 1;
                    else return;
                }; break;
            case XSL_OTHERWISE:
                {
                    if (status == 1) status = 2;
                    else return;
                }; break;
            default: return;
            }; 
            break;
        case XSL_FOR_EACH:
            if (hisop != XSL_SORT) return;
            break;
        case XSL_STYLESHEET:
        case XSL_TRANSFORM:
            if (hisop != XSL_WITH_PARAM) return;
            break;
        case XSL_TEMPLATE:
            if (hisop != XSL_PARAM) return;
            break;
        default: 
            return;
        }
    }
}

eFlag XSLElement::checkChildren()
{
    InstrTableItem &iData = instrTable[op];
    assert(iData.op == op);

    if (iData.flags & ELEM_CONT_PCDATA)
        return OK;

    situation.pushCurrV(this);
    if (!(iData.flags & ELEM_CONT) && contents.number())
        Err1(E_ELEM_MUST_EMPTY, xslOpNames[op]);

    int firstAfter = 0;
    if (iData.flags & ELEM_CONT_EXTRA)
        checkExtraChildren(firstAfter);

    for (int k = firstAfter; k < contents.number(); k++)
    {
        Vertex *w = contents[k];
        if (isText(w) || (isElement(w) && !isXSLElement(w)))
        {
            if (!(iData.flags & ELEM_CONT_INSTR))
                Err1(E_ELEM_CONT_TEXT_OR_LRE, xslOpNames[op]);
        }
        else
        {
            if (isXSLElement(w))
            {
                int hisflags = instrTable[toX(w) -> op].flags;
                if (!(((hisflags & ELEM_TOPLEVEL) && (iData.flags & ELEM_CONT_TOPLEVEL)) ||
                    ((hisflags & ELEM_INSTR) && (iData.flags & ELEM_CONT_INSTR))))
                    Err2(E_ELEM_CONTAINS_ELEM, xslOpNames[op], xslOpNames[toX(w) -> op]);
            }
            else
                Err1(E_BAD_ELEM_CONTENT, xslOpNames[op]);
        }
    };
    situation.popCurrent();
    return OK;
}

eFlag XSLElement::processChildren()
{
    switch(op)
    {
    case XSL_TEMPLATE:
        proc -> insertRule(this);
        break;
    };
    return OK;
}
